@@ -30,6 +30,9 @@ from base64 import b64encode, b64decode
 
 from zope.interface import implementer
 
+import txaio
+txaio.use_twisted()
+
 import twisted.internet.protocol
 from twisted.internet.defer import maybeDeferred
 from twisted.internet.interfaces import ITransport
@@ -37,11 +40,10 @@ from twisted.internet.error import ConnectionDone, ConnectionAborted, \
     ConnectionLost
 
 from autobahn.wamp import websocket
+from autobahn.websocket.types import ConnectionRequest, ConnectionResponse, \
+    ConnectionDeny
 from autobahn.websocket import protocol
-from autobahn.websocket import http
-from autobahn.twisted.util import peer2str
-
-from autobahn._logging import make_logger
+from autobahn.twisted.util import peer2str, transport_channel_id
 
 from autobahn.websocket.compress import PerMessageDeflateOffer, \
     PerMessageDeflateOfferAccept, \
@@ -77,8 +79,11 @@ class WebSocketAdapterProtocol(twisted.internet.protocol.Protocol):
     """
     Adapter class for Twisted WebSocket client and server protocols.
     """
+
     peer = '<never connected>'
 
+    log = txaio.make_logger()
+
     def connectionMade(self):
         # the peer we are connected to
         try:
@@ -90,6 +95,7 @@ class WebSocketAdapterProtocol(twisted.internet.protocol.Protocol):
             self.peer = peer2str(peer)
 
         self._connectionMade()
+        self.log.debug('Connection made to {peer}', peer=self.peer)
 
         # Set "Nagle"
         try:
@@ -100,34 +106,27 @@ class WebSocketAdapterProtocol(twisted.internet.protocol.Protocol):
 
     def connectionLost(self, reason):
         if isinstance(reason.value, ConnectionDone):
-            self.factory.log.debug("Connection to/from {peer} was closed cleanly",
-                                   peer=self.peer)
+            self.log.debug("Connection to/from {peer} was closed cleanly",
+                           peer=self.peer)
 
         elif isinstance(reason.value, ConnectionAborted):
-            self.factory.log.debug("Connection to/from {peer} was aborted locally",
-                                   peer=self.peer)
+            self.log.debug("Connection to/from {peer} was aborted locally",
+                           peer=self.peer)
 
         elif isinstance(reason.value, ConnectionLost):
-            # The following is ridiculous, but the treatment of reason.value.args
-            # across py2/3 and tx and over various corner cases is deeply fucked up.
+            message = str(reason.value)
             if hasattr(reason.value, 'message'):
                 message = reason.value.message
-            elif hasattr(reason.value, 'args') and type(reason.value.args) == tuple and len(reason.value.args) > 0:
-                message = reason.value.args[0]
-            else:
-                message = None
-
-            if message:
-                self.factory.log.debug("Connection to/from {peer} was lost in a non-clean fashion: {message}",
-                                       peer=self.peer, message=message)
-            else:
-                self.factory.log.debug("Connection to/from {peer} was lost in a non-clean fashion",
-                                       peer=self.peer)
+            self.log.debug(
+                "Connection to/from {peer} was lost in a non-clean fashion: {message}",
+                peer=self.peer,
+                message=message,
+            )
 
         # at least: FileDescriptorOverrun, ConnectionFdescWentAway - but maybe others as well?
         else:
-            self.factory.log.info("Connection to/from {peer} lost ({error_type}): {error})",
-                                  peer=self.peer, error_type=type(reason.value), error=reason.value)
+            self.log.debug("Connection to/from {peer} lost ({error_type}): {error})",
+                           peer=self.peer, error_type=type(reason.value), error=reason.value)
 
         self._connectionLost(reason)
 
@@ -178,8 +177,6 @@ class WebSocketAdapterProtocol(twisted.internet.protocol.Protocol):
         """
         Register a Twisted producer with this protocol.
 
-        Modes: Hybi, Hixie
-
         :param producer: A Twisted push or pull producer.
         :type producer: object
         :param streaming: Producer type.
@@ -201,15 +198,20 @@ class WebSocketServerProtocol(WebSocketAdapterProtocol, protocol.WebSocketServer
         res.addCallback(self.succeedHandshake)
 
         def forwardError(failure):
-            if failure.check(http.HttpException):
+            if failure.check(ConnectionDeny):
                 return self.failHandshake(failure.value.reason, failure.value.code)
             else:
-                if self.debug:
-                    self.factory._log("Unexpected exception in onConnect ['%s']" % failure.value)
-                return self.failHandshake(http.INTERNAL_SERVER_ERROR[1], http.INTERNAL_SERVER_ERROR[0])
+                self.log.debug("Unexpected exception in onConnect ['{failure.value}']", failure=failure)
+                return self.failHandshake("Internal server error: {}".format(failure.value), ConnectionDeny.INTERNAL_SERVER_ERROR)
 
         res.addErrback(forwardError)
 
+    def get_channel_id(self, channel_id_type=u'tls-unique'):
+        """
+        Implements :func:`autobahn.wamp.interfaces.ITransport.get_channel_id`
+        """
+        return transport_channel_id(self.transport, is_server=True, channel_id_type=channel_id_type)
+
 
 class WebSocketClientProtocol(WebSocketAdapterProtocol, protocol.WebSocketClientProtocol):
     """
@@ -219,12 +221,21 @@ class WebSocketClientProtocol(WebSocketAdapterProtocol, protocol.WebSocketClient
     def _onConnect(self, response):
         self.onConnect(response)
 
+    def startTLS(self):
+        self.log.debug("Starting TLS upgrade")
+        self.transport.startTLS(self.factory.contextFactory)
+
+    def get_channel_id(self, channel_id_type=u'tls-unique'):
+        """
+        Implements :func:`autobahn.wamp.interfaces.ITransport.get_channel_id`
+        """
+        return transport_channel_id(self.transport, is_server=False, channel_id_type=channel_id_type)
+
 
 class WebSocketAdapterFactory(object):
     """
     Adapter class for Twisted-based WebSocket client and server factories.
     """
-    log = make_logger("twisted")
 
 
 class WebSocketServerFactory(WebSocketAdapterFactory, protocol.WebSocketServerFactory, twisted.internet.protocol.ServerFactory):
@@ -293,17 +304,17 @@ class WrappingWebSocketAdapter(object):
     def onConnect(self, requestOrResponse):
 
         # Negotiate either the 'binary' or the 'base64' WebSocket subprotocol
-        if isinstance(requestOrResponse, protocol.ConnectionRequest):
+        if isinstance(requestOrResponse, ConnectionRequest):
             request = requestOrResponse
             for p in request.protocols:
                 if p in self.factory._subprotocols:
                     self._binaryMode = (p != 'base64')
                     return p
-            raise http.HttpException(http.NOT_ACCEPTABLE[0], "this server only speaks %s WebSocket subprotocols" % self.factory._subprotocols)
-        elif isinstance(requestOrResponse, protocol.ConnectionResponse):
+            raise ConnectionDeny(ConnectionDeny.NOT_ACCEPTABLE, u'this server only speaks {0} WebSocket subprotocols'.format(self.factory._subprotocols))
+        elif isinstance(requestOrResponse, ConnectionResponse):
             response = requestOrResponse
             if response.protocol not in self.factory._subprotocols:
-                self.failConnection(protocol.WebSocketProtocol.CLOSE_STATUS_CODE_PROTOCOL_ERROR, "this client only speaks %s WebSocket subprotocols" % self.factory._subprotocols)
+                self.failConnection(protocol.WebSocketProtocol.CLOSE_STATUS_CODE_PROTOCOL_ERROR, 'this client only speaks {0} WebSocket subprotocols'.format(self.factory._subprotocols))
             self._binaryMode = (response.protocol != 'base64')
         else:
             # should not arrive here
@@ -314,14 +325,13 @@ class WrappingWebSocketAdapter(object):
 
     def onMessage(self, payload, isBinary):
         if isBinary != self._binaryMode:
-            self.failConnection(protocol.WebSocketProtocol.CLOSE_STATUS_CODE_UNSUPPORTED_DATA, "message payload type does not match the negotiated subprotocol")
+            self.failConnection(protocol.WebSocketProtocol.CLOSE_STATUS_CODE_UNSUPPORTED_DATA, 'message payload type does not match the negotiated subprotocol')
         else:
             if not isBinary:
                 try:
                     payload = b64decode(payload)
                 except Exception as e:
-                    self.failConnection(protocol.WebSocketProtocol.CLOSE_STATUS_CODE_INVALID_PAYLOAD, "message payload base64 decoding error: {0}".format(e))
-            # print("forwarding payload: {0}".format(binascii.hexlify(payload)))
+                    self.failConnection(protocol.WebSocketProtocol.CLOSE_STATUS_CODE_INVALID_PAYLOAD, 'message payload base64 decoding error: {0}'.format(e))
             self._proto.dataReceived(payload)
 
     # noinspection PyUnusedLocal
@@ -329,7 +339,6 @@ class WrappingWebSocketAdapter(object):
         self._proto.connectionLost(None)
 
     def write(self, data):
-        # print("sending payload: {0}".format(binascii.hexlify(data)))
         # part of ITransport
         assert(type(data) == bytes)
         if self._binaryMode:
@@ -516,17 +525,17 @@ def connectWS(factory, contextFactory=None, timeout=30, bindAddress=None):
     else:
         from twisted.internet import reactor
 
+    if factory.isSecure:
+        if contextFactory is None:
+            # create default client SSL context factory when none given
+            from twisted.internet import ssl
+            contextFactory = ssl.ClientContextFactory()
+
     if factory.proxy is not None:
-        if factory.isSecure:
-            raise Exception("WSS over explicit proxies not implemented")
-        else:
-            conn = reactor.connectTCP(factory.proxy['host'], factory.proxy['port'], factory, timeout, bindAddress)
+        factory.contextFactory = contextFactory
+        conn = reactor.connectTCP(factory.proxy['host'], factory.proxy['port'], factory, timeout, bindAddress)
     else:
         if factory.isSecure:
-            if contextFactory is None:
-                # create default client SSL context factory when none given
-                from twisted.internet import ssl
-                contextFactory = ssl.ClientContextFactory()
             conn = reactor.connectSSL(factory.host, factory.port, factory, contextFactory, timeout, bindAddress)
         else:
             conn = reactor.connectTCP(factory.host, factory.port, factory, timeout, bindAddress)
@@ -581,9 +590,8 @@ class WampWebSocketServerFactory(websocket.WampWebSocketServerFactory, WebSocket
     def __init__(self, factory, *args, **kwargs):
 
         serializers = kwargs.pop('serializers', None)
-        debug_wamp = kwargs.pop('debug_wamp', False)
 
-        websocket.WampWebSocketServerFactory.__init__(self, factory, serializers, debug_wamp=debug_wamp)
+        websocket.WampWebSocketServerFactory.__init__(self, factory, serializers)
 
         kwargs['protocols'] = self._protocols
 
@@ -607,9 +615,8 @@ class WampWebSocketClientFactory(websocket.WampWebSocketClientFactory, WebSocket
     def __init__(self, factory, *args, **kwargs):
 
         serializers = kwargs.pop('serializers', None)
-        debug_wamp = kwargs.pop('debug_wamp', False)
 
-        websocket.WampWebSocketClientFactory.__init__(self, factory, serializers, debug_wamp=debug_wamp)
+        websocket.WampWebSocketClientFactory.__init__(self, factory, serializers)
 
         kwargs['protocols'] = self._protocols
 
