@@ -36,11 +36,15 @@ import math
 import random
 from datetime import datetime, timedelta
 from pprint import pformat
+from array import array
+
+import six
 
 import txaio
 
 
-__all__ = ("utcnow",
+__all__ = ("xor",
+           "utcnow",
            "utcstr",
            "id",
            "rid",
@@ -50,7 +54,42 @@ __all__ = ("utcnow",
            "Tracker",
            "EqualityMixin",
            "ObservableMixin",
-           "IdGenerator")
+           "IdGenerator",
+           "generate_token",
+           "generate_activation_code",
+           "generate_serial_number",
+           "generate_user_password")
+
+
+def xor(d1, d2):
+    """
+    XOR two binary strings of arbitrary (equal) length.
+
+    :param d1: The first binary string.
+    :type d1: binary
+    :param d2: The second binary string.
+    :type d2: binary
+
+    :returns: XOR(d1, d2)
+    :rtype: binary
+    """
+    if type(d1) != six.binary_type:
+        raise Exception("invalid type {} for d1 - must be binary".format(type(d1)))
+    if type(d2) != six.binary_type:
+        raise Exception("invalid type {} for d2 - must be binary".format(type(d2)))
+    if len(d1) != len(d2):
+        raise Exception("cannot XOR binary string of differing length ({} != {})".format(len(d1), len(d2)))
+
+    d1 = array('B', d1)
+    d2 = array('B', d2)
+
+    for i in range(len(d1)):
+        d1[i] ^= d2[i]
+
+    if six.PY3:
+        return d1.tobytes()
+    else:
+        return d1.tostring()
 
 
 def utcstr(ts=None):
@@ -212,6 +251,100 @@ def newid(length=16):
     return base64.b64encode(os.urandom(l))[:length].decode('ascii')
 
 
+# a standard base36 character set
+# DEFAULT_TOKEN_CHARS = string.digits + string.ascii_uppercase
+
+# we take out the following 9 chars (leaving 27), because there
+# is visual ambiguity: 0/O/D, 1/I, 8/B, 2/Z
+DEFAULT_TOKEN_CHARS = u'345679ACEFGHJKLMNPQRSTUVWXY'
+"""
+Default set of characters to create rtokens from.
+"""
+
+DEFAULT_ZBASE32_CHARS = u'13456789abcdefghijkmnopqrstuwxyz'
+"""
+http://philzimmermann.com/docs/human-oriented-base-32-encoding.txt
+
+Our choice of confusing characters to eliminate is: `0', `l', `v', and `2'.  Our
+reasoning is that `0' is potentially mistaken for `o', that `l' is potentially
+mistaken for `1' or `i', that `v' is potentially mistaken for `u' or `r'
+(especially in handwriting) and that `2' is potentially mistaken for `z'
+(especially in handwriting).
+
+Note that we choose to focus on typed and written transcription more than on
+vocal, since humans already have a well-established system of disambiguating
+spoken alphanumerics, such as the United States military's "Alpha Bravo Charlie
+Delta" and telephone operators' "Is that 'd' as in 'dog'?".
+"""
+
+
+def generate_token(char_groups, chars_per_group, chars=None, sep=None, lower_case=False):
+    """
+    Generate cryptographically strong tokens, which are strings like `M6X5-YO5W-T5IK`.
+    These can be used e.g. for used-only-once activation tokens or the like.
+
+    The returned token has an entropy of:
+
+       math.log(len(chars), 2.) * chars_per_group * char_groups
+
+    bits. With the default charset and 4 characters per group, rtoken() produces
+    tokens with the following entropy:
+
+        character groups    entropy (at least)  recommended use
+
+        2                    38 bits
+        3                    57 bits            one-time activation or pairing code
+        4                    76 bits            secure user password
+        5                    95 bits
+        6                   114 bits            globally unique serial / product code
+        7                   133 bits
+
+    Here are 3 examples:
+
+        * token(3): 9QXT-UXJW-7R4H
+        * token(4): LPNN-JMET-KWEP-YK45
+        * token(6): NXW9-74LU-6NUH-VLPV-X6AG-QUE3
+
+    :param char_groups: Number of character groups (or characters if chars_per_group == 1).
+    :type char_groups: int
+    :param chars_per_group: Number of characters per character group (or 1 to return a token with no grouping).
+    :type chars_per_group: int
+    :param chars: Characters to choose from. Default is 27 character subset
+        of the ISO basic Latin alphabet (see: DEFAULT_TOKEN_CHARS).
+    :type chars: unicode or None
+    :param sep: When separating groups in the token, the separater string.
+    :type sep: unicode
+
+    :returns: The generated token.
+    :rtype: unicode
+    """
+    assert(type(char_groups) in six.integer_types)
+    assert(type(chars_per_group) in six.integer_types)
+    assert(chars is None or type(chars) == six.text_type)
+    chars = chars or DEFAULT_TOKEN_CHARS
+    if lower_case:
+        chars = chars.lower()
+    sep = sep or u'-'
+    rng = random.SystemRandom()
+    token_value = u''.join(rng.choice(chars) for _ in range(char_groups * chars_per_group))
+    if chars_per_group > 1:
+        return sep.join(map(u''.join, zip(*[iter(token_value)] * chars_per_group)))
+    else:
+        return token_value
+
+
+def generate_activation_code():
+    return generate_token(char_groups=3, chars_per_group=4, chars=DEFAULT_TOKEN_CHARS, sep=u'-', lower_case=False)
+
+
+def generate_user_password():
+    return generate_token(char_groups=16, chars_per_group=1, chars=DEFAULT_ZBASE32_CHARS, sep=u'-', lower_case=True)
+
+
+def generate_serial_number():
+    return generate_token(char_groups=6, chars_per_group=4, chars=DEFAULT_TOKEN_CHARS, sep=u'-', lower_case=False)
+
+
 # Select the most precise walltime measurement function available
 # on the platform
 #
